
/**
 * Ein Objekt dieer Klasse stellt ein ein gerastetertes Feld auf der Zeichenflaeche dar.
 * 
 * @author Klaus Dingemann
 * @version 02.02.2018
 */
import sas.*;
import java.awt.Color;
public class Rasterflaeche
{
   
    private int anzahlKachelnX;
    private int anzahlKachelnY;
    private int pixelBreite;
    private int pixelHoehe;
    private int kachelBreite;
    private int kachelHoehe;
    private Rectangle hintergrund;
    private Kachel[][] kacheln;
    
    /**
     * Erzeugt eine aus Kachelobjekten bestehende Rasterfläche. Als Parameter werden die Anzahl der 
     * Kacheln in jeder Zeile und in jeder Spalte, die Breite und die Höhe der Rasterfläche in Pixeln
     * sowie der Farbe der Kacheln angegeben. Die Breite und Höhe der Kacheln wird daraus berechnet. Die
     * Kacheln werden durch graue Linien getrennt dargestellt.
     * @param pAnzahlKachelnX; Anzahl der Spalten der Rasterfläche
     * @param pAnzahlKachelnY: Anzahl der Zeilen der Rasterfläche
     * @param pPixelBreite: Breite der Rasterfläche in Pixeln
     * @param pPixelHoehe: Breite der Rasterfläche in Pixeln
     * @param pKachelFarbe: Farbe aller Kacheln der Rasterfläche
     */
    public Rasterflaeche(int pAnzahlKachelnX, int pAnzahlKachelnY, double pPixelBreite, double pPixelHoehe, 
           Color pKachelfarbe)
    {
       anzahlKachelnX = pAnzahlKachelnY;
       anzahlKachelnY = pAnzahlKachelnY;
       kachelBreite = (int)pPixelBreite / pAnzahlKachelnX;
       kachelHoehe = (int)pPixelHoehe / pAnzahlKachelnY;
       pixelBreite = kachelBreite * anzahlKachelnX;
       pixelHoehe = kachelHoehe * anzahlKachelnY;
       hintergrund = new Rectangle(0, 0, pixelBreite, pixelHoehe, new Color(200, 200, 200));
       kacheln = new Kachel[anzahlKachelnX][anzahlKachelnY];
       for (int i = 0; i<anzahlKachelnX ;i++) {
           for (int j = 0; j < anzahlKachelnY; j++) {
               kacheln[i][j] = new Kachel(i,j,kachelBreite,kachelHoehe, pKachelfarbe);
            }
        }
    }
     /**
     * Die Methode stellt die Rasterfläche mit Schachbrettemuster dar. 
     * @param pFarbe: Zweite Kachelfarbe
     */
    
    public void schachBrettMuster(Color pFarbe) {
           for (int i = 0; i< anzahlKachelnX ;i++) {
           for (int j = 0; j < anzahlKachelnY; j++) {
               if (i % 2 == 0 && j % 2  == 0 || i % 2 == 1 && j % 2 == 1) {
                   kacheln[i][j].setzeFarbe(pFarbe);
                }
            }
        }
    }
    
    /**
     * Umrechnung einer x-Kachelkoordinate in die
     * x-Pixelkoordinate der Zeichenflaeche. Die x-Koordinate
     * der linken oberen Ecke der Kachel auf der Zeichenfläche
     * wird zurückgegeben.
     * @param pX: X-Koordinate der Rasterfläche
     */
    public int kachelXInPixelX(int pX) {
       return pX*kachelBreite+1;
    }
      
     /**
     * Umrechnung einer y-Kachelkoordinate in die
     * x-Pixelkoordinate der Zeichenflaeche. Die x-Koordinate
     * der linken oberen Ecke der Kachel auf der Zeichenfläche
     * wird zurückgegeben.
     * @param pY: Y-Koordinate der Rasterfläche
     */
    public int kachelYInPixelY(int pY) {
       return pY*kachelHoehe+1;
    }
    
    /**
     * Umrechnung einer x-Koordinate der Zeichenfläche
     * in die zugehörige x-Koordinate der Rasterfläche.
     * @param pX: X-Pixelkoordinate
     * 
     */
    public int pixelXInKachelX(double pX) {
       return (int)pX / kachelBreite;
    }
        
    /**
     * Umrechnung einer y-Pixel-Koordinate der Zeichenfläche
     * in die zugehörige x-Koordinate der Rasterfläche.
     * @param pY: Y-Pixelkoordinate
     */
    public int pixelYInKachelY(double pY) {
       return (int)pY / kachelHoehe;
    }
    
    /**
     * Setzt ein Hindernis mit der angegebenen Farbe in die Rasterfläche
     * @param pX: x-Koordinate des Hindernisses
     * @param pY: y-Koordinate des Hindernisses
     */
    public void setzeHindernis(int pX, int pY, Color pColor) {
        if (pX>=0 && pX < anzahlKachelnX && pY >= 0 && pY < anzahlKachelnY) {
            kacheln[pX][pY].setzteHindernis(pColor);
        }
    }
    /**
     * Gibt true zurück, wenn sich auf der Kachel mit den angegebenen
     * Rasterkoordinaten ein Hindernis befindet.
     * @param pX: x-Koordinate 
     * @param pY: y-Koordinate 
     */
    public boolean istHindernis(int pX, int pY) {
        return kacheln[pX][pY].istHindernis();
    }
    
    /**
     * Gibt true zurück, die übergebenen Koordinaten, einem Feld der Rasterfläche
     * gehören.
     * @param pX: x-Koordinate 
     * @param pY: y-Koordinate 
     */
    public boolean istInnen(int pX, int pY) {
        return pX>=0 && pX<anzahlKachelnX && pY>=0 && pY<anzahlKachelnY;
    }
    
    /**
     * liefert die Breite einer Kachel in Pixel.
     */
    public int gibKachelBreite() {
        return kachelBreite;
    }
    
     /**
     * liefert die Höhe einer Kachel in Pixel.
     */
    public int gibKachelHoehe(){
        return kachelHoehe;
    }
    
    /**Liefert eine Referenz auf das Kachelobjekt mit den
     * angegebenen Koordinaten.
     */
    public Kachel gibKachel(int pX, int pY) {
        return kacheln[pX][pY];
    }
    
}  
    
