
/**
 * Ein Objekt dieser Klasse ist das Spielfeld des Spiels Schatzsuche.
 * 
 * @author Klaus Dingemann 
 * @version 18.01.2018
 */
import sas.*;
import java.awt.Color;
public class Schatzkammer extends Rasterflaeche
{
    private final int ANZAHLHINDERNISSE = 28;
    private final int MAXSCHAETZE = 15;
    private Circle[] schaetze = new Circle[MAXSCHAETZE];
    private int anzahlSchaetze;
    private int zeilen;
    private int spalten;
    
   
    /**
     * Konstruktor zur Erzeugung des Spielfelds mit den Hindernissen als Rasterfläche
     * @param pSpalten: Anzahl der Spalten des Spielfeldes
     * @param pZeilen: Anzahl der Zeilen des Spielfeldes
     * @param pBreite: Breite des Spielfeldes in Pixel
     * @param pHoehe: Höhe des Spielfeldes in Pixel
     */
    public Schatzkammer(int pSpalten, int pZeilen, int pBreite, int pHoehe)
    {
       super(pSpalten, pZeilen, pBreite, pHoehe, Color.YELLOW); 
       zeilen = pZeilen;
       spalten = pSpalten;
       erzeugeHindernisse();
       erzeugeSchaetze();
    }

    /**
     * Erzeugt ANZAHLHINDERNISSE an zufälliger Position
     */
    private void erzeugeHindernisse(){
        int anzahl = 0;
        while (anzahl < ANZAHLHINDERNISSE){
           int  x = Tools.randomNumber(0,spalten - 1);
           int y = Tools.randomNumber(1, zeilen - 2);
           if (!this.istHindernis(x, y)) {
               this.setzeHindernis(x, y, Color.RED);
               anzahl++;
           }
        }    
    }
    
    /**
     * Erzeugt MAXSCHAETZE. Einen pro Zeile in zufälliger Spalte.
     */
    private void erzeugeSchaetze(){
        anzahlSchaetze = 0;
        while (anzahlSchaetze < MAXSCHAETZE) {
           int  sx = Tools.randomNumber(0,spalten - 1);
           int vx = this.kachelXInPixelX(sx)+gibKachelBreite() / 2 - 4;
           int vy = this.kachelYInPixelY(anzahlSchaetze)+gibKachelHoehe() / 2 - 4;
           if (!this.gibKachel(sx,anzahlSchaetze).istHindernis()) {
               schaetze[anzahlSchaetze] = new Circle(vx, vy, 4, Color.GREEN);
               anzahlSchaetze++;
           }
        }      
    }
    
    /**
     * Entfernt den Schatz an der angegebenen Position.
     * @param pX: X-Position
     * @param pY: Y-Position
     */
    public  void entferneSchatz(int pX, int pY){
      boolean gefunden = false;
      int zaehler = 0;
      while (!gefunden && zaehler<MAXSCHAETZE){
          int sx = pixelXInKachelX(schaetze[zaehler].getShapeX());
          int sy = pixelYInKachelY(schaetze[zaehler].getShapeY());
          gefunden = sx == pX && sy == pY;
          if (gefunden && !schaetze[zaehler].getHidden()) {
              schaetze[zaehler].setHidden(true);
              anzahlSchaetze--;
          } else {
              zaehler++;
            }
        }
    }
    

   /**
    * Liefert die Anazhl der noch vorhandenen Schätze
    */ 
   public int gibAnzahlSchaetze(){
       return anzahlSchaetze;
    }
        
}