
/**
 * Ein Objekt dieser klasse ist ein Spielfeld für das Spiel
 * Wolf und Schafe. Das Spielfeld überprüft die Korrektheit 
 * der Züge und überprüft das Spielende.
 * 
 * @author Klaus Dingemann 
 * @version 29.01.2018
 */
import sas.*;
import java.awt.Color;
public class Spielfeld extends Rasterflaeche
{
    Spielstein steinWolf;
    Spielstein[] schafe = new Spielstein[4];
     
   
    /**
     * Konstruktor für Objekte der Klasse Spielfeld
     */

    public Spielfeld()
    {
       super(8, 8, 400, 400, Color.WHITE);
       this.schachBrett(Color.BLACK);
       steinWolf = new Spielstein(this, 4, 7, Color.RED);
       schafe[0] = new Spielstein(this, 1, 0, Color.YELLOW);
       schafe[1] = new Spielstein(this, 3, 0, Color.YELLOW);
       schafe[2] = new Spielstein(this, 5, 0, Color.YELLOW);
       schafe[3] = new Spielstein(this, 7, 0, Color.YELLOW);
    }

    public boolean feldBelegt(Position pPosition) {
      boolean mitSchafBelegt = false;
      for (int i=0; i<4; i++) {
          mitSchafBelegt = mitSchafBelegt || pPosition.istGleich(schafe[i].gibPosition());
      }
      return pPosition.istGleich(steinWolf.gibPosition()) || mitSchafBelegt;
    }
    
    public boolean imFeld(Position pPosition) {
        return this.istInnen(pPosition.gibXPosition(), pPosition.gibYPosition());
    }
    
    public boolean zugZulaessig (Spielstein pSpielstein,Position pNeuePosition,  boolean pIstSchaf) {
        if (pIstSchaf) {
          int x = pNeuePosition.gibXPosition();
          int y = pNeuePosition.gibYPosition();
          boolean innen = this.istInnen(x, y); 
          boolean zugRegelX = Math.abs(pNeuePosition.gibXPosition()-pSpielstein.gibPosition().gibXPosition())==1; 
          boolean zugRegelY= pNeuePosition.gibYPosition()-pSpielstein.gibPosition().gibYPosition()==1; 
          return innen && zugRegelX && zugRegelY && !feldBelegt(pNeuePosition);
        } else { 
            return  this.istInnen(pNeuePosition.gibXPosition(), pNeuePosition.gibYPosition()) &&
                    Math.abs(pNeuePosition.gibXPosition()-pSpielstein.gibPosition().gibXPosition())==1 &&
                    Math.abs(pNeuePosition.gibYPosition()-pSpielstein.gibPosition().gibYPosition())==1 && 
                    !feldBelegt(pNeuePosition);
        }
    }    
    
    public void zieheNach (Spielstein pSpielstein,Position pNeuePosition) {
        pSpielstein.bewegeAuf(pNeuePosition);     
    }
    
    public Spielstein[] gibSchafe() {
        return schafe;
    }
    
    public Spielstein gibWolf() {
       return steinWolf;
    }
    
    public int spielBeendet(){
        int xPosition = steinWolf.gibPosition().gibXPosition();
        int yPosition = steinWolf.gibPosition().gibYPosition();
        boolean zug1zulaessig = zugZulaessig(steinWolf, new Position(xPosition+1, yPosition+1),false);
        boolean zug2zulaessig = zugZulaessig(steinWolf, new Position(xPosition+1, yPosition-1),false);
        boolean zug3zulaessig = zugZulaessig(steinWolf, new Position(xPosition-1, yPosition+1),false);
        boolean zug4zulaessig = zugZulaessig(steinWolf, new Position(xPosition-1, yPosition-1),false);
        boolean wolfKannNichtZiehen = !(zug1zulaessig || zug2zulaessig || zug3zulaessig || zug3zulaessig);
        int minSchafPosition = schafe[0].gibPosition().gibYPosition();
        for (int i = 1; i<4; i++) {
            if (schafe[i].gibPosition().gibYPosition() < minSchafPosition) {
                minSchafPosition = schafe[i].gibPosition().gibYPosition();
            }
        }
        boolean wolfDurchgebrochen=false;
        if (steinWolf.gibPosition().gibYPosition() <= minSchafPosition) {
            wolfDurchgebrochen = true;
        } else {
            wolfDurchgebrochen = false;
        }
        if (wolfKannNichtZiehen) {
            return 1;
        } else if (wolfDurchgebrochen) {
            return 2;
        } else {
            return 0;
        }
    }
}


